(********************************************************************************)
(*  ocamlcore-api: Access to OCamlCore.org services                             *)
(*                                                                              *)
(*  Copyright (C) 2011, OCamlCore SARL                                          *)
(*                                                                              *)
(*  This library is free software; you can redistribute it and/or modify it     *)
(*  under the terms of the GNU Lesser General Public License as published by    *)
(*  the Free Software Foundation; either version 2.1 of the License, or (at     *)
(*  your option) any later version, with the OCaml static compilation           *)
(*  exception.                                                                  *)
(*                                                                              *)
(*  This library is distributed in the hope that it will be useful, but         *)
(*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  *)
(*  or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more          *)
(*  details.                                                                    *)
(*                                                                              *)
(*  You should have received a copy of the GNU Lesser General Public License    *)
(*  along with this library; if not, write to the Free Software Foundation,     *)
(*  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA               *)
(********************************************************************************)

(** Handle common data for OCAWeb
    @author Sylvain Le Gall
  *)

open Common
open Eliom_sessions
open XHTML.M
open Eliom_predefmod.Xhtml
open Simplexmlparser

type t =
    {
      web_ctxt:           OCAContext.t;
      web_logout:         server_params -> uri;
      web_login:          server_params -> uri;
      web_with_login:     server_params -> uri -> uri;
      web_logout_get:     server_params -> uri;
      web_login_get:      server_params -> string -> string -> uri;
      web_my_account:     server_params -> uri;
      web_new_account:    server_params -> uri;
      web_manage_account: server_params -> uri;
      web_lost_passwd:    server_params -> uri;
      web_redirect:       server_params -> string -> string;
      web_ignore_ip:      bool;

      web_my_account_ext:     server_params -> uri;
      web_new_account_ext:    server_params -> uri;
      web_manage_account_ext: server_params -> uri;
    }

let create 
      ?my_account 
      ?new_account 
      ?manage_account
      ?(stub=false) 
      ?conf_ini 
      ?(ignore_ip=false) 
      ?(redirect=(fun _ s -> s))
      ?(base_path=[])
      () = 

  let ctxt = 
    if stub then 
      begin
        let ctxt, stub_id = 
          OCAContext.create_stub ()
        in
          OCAStub.populate stub_id;
          ctxt
      end
    else
      begin
        OCAContext.create ?conf_ini ()
      end
  in

  let srvc_tmpl ?uri_param srvc sp = 
    make_uri ~service:srvc ~sp 
      (Some 
         (match uri_param with 
            | Some uri -> string_of_uri uri
            | None -> self_uri sp))
  in

  let res = 
    match ctxt with 
      | `Live _ ->
          begin
            let base_uri = 
              "https://forge.ocamlcore.org/"
            in
            let uri_tmpl ?extra ?uri_param str = 
              let str' = 
                base_uri^str
              in
                uri_template ?extra ?uri_param redirect str'
            in
            let my_account     = uri_tmpl "my/" in
            let new_account    = uri_tmpl "account/register.php" in
            let manage_account = uri_tmpl "account/" in
            let login          = "account/login.php?return_to=$(redirect)" in
              {
                web_ctxt               = ctxt;
                web_logout             = uri_tmpl "account/logout.php";
                web_login              = uri_tmpl login;
                web_with_login         = (fun sp uri_param -> 
                                            uri_tmpl ~uri_param login sp);
                web_logout_get         = uri_tmpl "account/logout.php";
                web_my_account         = my_account;
                web_new_account        = new_account;
                web_manage_account     = manage_account;
                web_lost_passwd        = uri_tmpl "account/lostpw.php";
                web_redirect           = redirect;
                web_ignore_ip          = ignore_ip;
                web_my_account_ext     = my_account;
                web_new_account_ext    = new_account;
                web_manage_account_ext = manage_account;
                web_login_get = 
                  (fun sp login passwd ->
                     uri_tmpl ~extra:["login", login; "password", passwd]
                       "account/login.php?\
                        form_pw=$(password)&\
                        form_loginname=$(login)"
                       sp);
              }
          end

      | `Stub stub_id ->
          begin
            let logout, logout_get = 
              SessionStub.logout ~ctxt 
                (SessionLow.get ~ctxt ignore_ip) 
                base_path
            in
            let login, login_get = 
              SessionStub.login ~ctxt 
                base_path
            in
            let my_account = 
              srvc_tmpl (AccountStub.my_account ~ctxt base_path)
            in
            let new_account =
              let service = 
                AccountStub.new_account ~ctxt base_path
              in
                fun sp -> make_uri ~service ~sp ()
            in
            let manage_account =
              srvc_tmpl (AccountStub.manage_account ~ctxt base_path)
            in
            let lost_passwd =
              srvc_tmpl (AccountStub.lost_passwd ~ctxt base_path)
            in
            let login_get sp login passwd =
              make_uri ~service:login_get ~sp (login, passwd)
            in
            let logout_get sp = 
              make_uri ~service:logout_get ~sp ()
            in
              {
                web_ctxt               = ctxt;
                web_logout             = srvc_tmpl logout;
                web_login              = srvc_tmpl login;
                web_with_login         = (fun sp uri_param ->
                                            srvc_tmpl ~uri_param login sp);
                web_logout_get         = logout_get;
                web_login_get          = login_get;
                web_my_account         = my_account;
                web_new_account        = new_account;
                web_manage_account     = manage_account;
                web_lost_passwd        = lost_passwd;
                web_redirect           = redirect;
                web_ignore_ip          = ignore_ip;
                web_my_account_ext     = my_account;
                web_new_account_ext    = new_account;
                web_manage_account_ext = manage_account;
              }
          end
  in
    (* Make a difference for overloaded services *)
  let res = 
    let overload_if_set srvc_opt vl = 
      match srvc_opt with 
        | Some srvc ->
            srvc_tmpl srvc
        | None ->
            vl
    in
      {res with
           web_my_account = 
             overload_if_set my_account res.web_my_account;
           web_new_account = 
             overload_if_set new_account res.web_new_account;
           web_manage_account = 
             overload_if_set manage_account res.web_manage_account}
  in

    res

let create_from_config ?my_account ?new_account ?manage_account () = 
  let stub = 
    ref false
  in
  let conf_ini =
    ref None
  in
  let ignore_ip = 
    ref None
  in
  let redirect = 
    ref None
  in
  let base_path =
    ref None
  in
  let rec parse_config = 
    function
      | Element ("base-path", _, [PCData str]) ->
          base_path := Some (BatString.nsplit str "/")

      | Element ("stub", _, [PCData str]) ->
          stub := bool_of_string str

      | Element ("ini", _, [PCData fn]) ->
          conf_ini := Some fn

      | Element ("ignore-ip", _, [PCData str]) ->
          ignore_ip := Some (bool_of_string str)

      | Element ("redirect", attrs, []) ->
          let replace = 
            List.assoc "replace" attrs
          in
          let by =
            List.assoc "by" attrs
          in
          let fredirect _ str =
            snd (BatString.replace ~str ~sub:replace ~by)
          in
            redirect := Some fredirect

      | Element (nm, _, _) ->
          failwith 
            (Printf.sprintf
               "Don't know what to do with 
                configuration element '%s'"
               nm)

      | PCData str ->
          failwith 
            (Printf.sprintf
               "Don't know what to do with 
                configuration pcdata '%s'"
               str)
  in

  let rec parse_config_top =
    function 
      | (Element ("ocamlcore-api", _, lst)) :: _ ->
          List.iter parse_config lst 
      | _ :: tl ->
          parse_config_top tl
      | [] ->
          ()
  in

  let () = 
    parse_config_top (Eliom_sessions.get_config ())
  in

    create 
      ?my_account 
      ?new_account 
      ?manage_account
      ~stub:!stub 
      ?conf_ini:!conf_ini 
      ?ignore_ip:!ignore_ip
      ?redirect:!redirect
      ?base_path:!base_path
      ()

let to_oca_context t =
  t.web_ctxt
